@@ -1,8 +1,8 @@
-""" # noqa: D205
-Representation of CHGCAR files
-------------------------------
+"""
+Representation of CHGCAR files.
 
-Charge density data node (stores CHGCAR files).
+-------------------------------
+Charge density data node (stores CHGCAR files in the repository).
 """
 # pylint: disable=abstract-method
 # explanation: pylint wrongly complains about (aiida) Node not implementing query
