@@ -7,7 +7,7 @@ The file parser that handles the parsing of DOSCAR files.
 # pylint: disable=unsubscriptable-object  # pylint/issues/3139
 import numpy as np
 
-from aiida_vasp.parsers.node_composer import NodeComposer
+from aiida_vasp.parsers.node_composer import NodeComposer, get_node_composer_inputs_from_file_parser
 from aiida_vasp.parsers.file_parsers.parser import BaseFileParser
 
 # Map from number of columns in DOSCAR to dtype.
@@ -42,7 +42,6 @@ class DosParser(BaseFileParser):
     def __init__(self, *args, **kwargs):
         super(DosParser, self).__init__(*args, **kwargs)
         self._dos = None
-        self.init_with_kwargs(**kwargs)
 
     def _parse_file(self, inputs):
         """Read a VASP DOSCAR file and extract metadata and a density of states data array."""
@@ -130,6 +129,6 @@ class DosParser(BaseFileParser):
     @property
     def dos(self):
         if self._dos is None:
-            composer = NodeComposer(file_parsers=[self])
-            self._dos = composer.compose('array', quantities=['doscar-dos'])
+            inputs = get_node_composer_inputs_from_file_parser(self, quantity_keys=['doscar-dos'])
+            self._dos = NodeComposer.compose('array', inputs)
         return self._dos
