@@ -5,7 +5,7 @@ WAVECAR parser.
 The file parser that handles the parsing of WAVECAR files.
 """
 from aiida_vasp.parsers.file_parsers.parser import BaseFileParser
-from aiida_vasp.parsers.node_composer import NodeComposer
+from aiida_vasp.parsers.node_composer import NodeComposer, get_node_composer_inputs_from_file_parser
 
 
 class WavecarParser(BaseFileParser):
@@ -22,7 +22,6 @@ class WavecarParser(BaseFileParser):
     def __init__(self, *args, **kwargs):
         super(WavecarParser, self).__init__(*args, **kwargs)
         self._wavecar = None
-        self.init_with_kwargs(**kwargs)
 
     def _parse_file(self, inputs):
         """Create a DB Node for the WAVECAR file."""
@@ -39,6 +38,6 @@ class WavecarParser(BaseFileParser):
     @property
     def wavecar(self):
         if self._wavecar is None:
-            composer = NodeComposer(file_parsers=[self])
-            self._wavecar = composer.compose('vasp.wavefun')
+            inputs = get_node_composer_inputs_from_file_parser(self)
+            self._wavecar = NodeComposer.compose('vasp.wavefun', inputs)
         return self._wavecar
