@@ -9,8 +9,10 @@ or not currently checked in it. This workchain does currently nothing.
 
 # pylint: disable=attribute-defined-outside-init
 from aiida.common.extendeddicts import AttributeDict
+from aiida.common.exceptions import NotExistent
 from aiida.engine import WorkChain, while_, append_
 from aiida.plugins import WorkflowFactory
+
 from aiida_vasp.utils.aiida_utils import get_data_class, get_data_node
 from aiida_vasp.utils.workchains import prepare_process_inputs, compose_exit_code
 
@@ -91,7 +93,7 @@ class VerifyWorkChain(WorkChain):
         self.ctx.inputs.update(self.exposed_inputs(self._next_workchain))
 
         # Make sure we do not have any floating dict (convert to Dict)
-        self.ctx.inputs = prepare_process_inputs(self.ctx.inputs)
+        self.ctx.inputs = prepare_process_inputs(self.ctx.inputs, namespaces=['dynamics'])
 
     def run_next_workchain(self):
         """Run the next workchain."""
@@ -129,6 +131,12 @@ class VerifyWorkChain(WorkChain):
             self.ctx.exit_code = compose_exit_code(next_workchain_exit_status, next_workchain_exit_message)
             self.report('The called {}<{}> returned a non-zero exit status. '
                         'The exit status {} is inherited'.format(workchain.__class__.__name__, workchain.pk, self.ctx.exit_code))
+            # Make sure at the very minimum we attach the misc node (if it exists) that contains notifications and other
+            # quantities that can be salvaged
+            try:
+                self.out('misc', workchain.outputs['misc'])
+            except NotExistent:
+                pass
 
         return self.ctx.exit_code
 
